﻿--[[---------------------------------------------------------------------------
  ColaMachine, a module for sharing data between ColaLight users
  
  Copyright (C) 2006  Murazorz
  
  Part of the code is influenced by VendorValues Copyright by Saeris
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
  02110-1301, USA.
  
-------------------------------------------------------------------------------

  This is a data sharing module for ColaLight. It allows for broadcasting 
  the ColaLight value database over either a custom channel or the built-in
  addon channel. It also adds any data received through the same channel.
  All data is accepted, but only overwrites existing data if the sender is
  in the trusted user list.

  Any bugs, questions and requests can be posted in the 
  http://www.wowace.com/forums/index.php?topic=2351.0 thread.
-----------------------------------------------------------------------------]]

--[[
Name: ColaMachine
Revision: $Revision: %%$
Author(s):  Murazorz (otnikion@gmail.com)
Website: http://www.wowace.com/forums/index.php?topic=2351.0
Documentation: http://www.wowace.com/wiki/ColaMachine
Subversion: http://svn.wowace.com/wowace/trunk/ColaMachine/
Issues and feature requests: http://www.wowace.com/forums/index.php?topic=2351.0
Description: A module for sharing data between ColaLight users.
Dependencies: ColaLight
]]

assert(ColaLight, "ColaLight not found!")

local tonumber = tonumber
local tostring = tostring
local strgmatch = string.gmatch
local strformat = string.format
local strlen = string.len

local ColaLight = ColaLight

local L = AceLibrary("AceLocale-2.2"):new("ColaMachine")

L:RegisterTranslations("enUS", function()
	return {
        ["ColaMachine"] = true,
        ["ColaMachine - blows ColaLight cans to pieces"] = true,
        ["trust"] = true,
        ["Add trusted user"] = true,
        ["<name>, <<guild>> adds all in the guild to your trust list"] = true,
        ["delete"] = true,
        ["Delete trusted user"] = true,
        ["<name>, <<guild>> deletes all in the guild from your trust list"] = true,
        ["list"] = true,
        ["List trusted users"] = true,
        ["channel"] = true,
        ["Set the data synchronization channel"] = true,
        ["<channel>, <<default>> sets the channel to the default channel, <<addon>> sets the channel to the built-in add on channel"] = true,
        ["broadcast"] = true,
        ["Broadcast the whole database to the designated channel"] = true,
        ["Updated version, cleared the trust list, you need to re-enter users"] = true,
        ["Added the whole guild to the trusted user list"] = true,
        ["Added trusted user: "] = true,
        ["Removed the whole guild from the trusted user list"] = true,
        ["Removed trusted user: "] = true,
        ["Trusting the whole guild"] = true,
        ["Trusted Users: "] = true,
        ["Using the built-in add on channel"] = true,
        ["Setting data synchronization channel to default: "] = true,
        ["Removing use of add on channel"] = true,
        ["Setting data synchronization channel to the built-in add on channel"] = true,
        ["Setting data synchronization channel to: "] = true,
        ["Broadcasting the value database..."] = true,
        ["Configured to use the add on channel, but you're not in a guild, use a custom channel instead"] = true,
        ["Broadcasting already in progress."] = true,
        ["Couldn't join the proper channel for data synchronization"] = true,
	}
end)

L:RegisterTranslations("deDE", function()
	return {
        ["ColaMachine"] = "ColaMachine",
        ["ColaMachine - blows ColaLight cans to pieces"] = "ColaMachine - blows ColaLight cans to pieces",
        ["trust"] = "vertrauen",
        ["Add trusted user"] = "Vertrauensw\195\188rdigen Benutzer hinzuf\195\188gen",
        ["<name>, <<guild>> adds all in the guild to your trust list"] = "<name>, <<guild>> f\195\188gt alle in deiner Gilde zur Vertrauensliste hinzu",
        ["delete"] = "l\195\182schen",
        ["Delete trusted user"] = "Vertrauensw\195\188rdigen Benutzer l\195\182schen",
        ["<name>, <<guild>> deletes all in the guild from your trust list"] = "<name>, <<guild>> l\195\182scht alle in deiner Gilde von der Vertrauensliste",
        ["list"] = "auflisten",
        ["List trusted users"] = "Vertrauensw\195\188rdige Benutzer auflisten",
        ["channel"] = "channel",
        ["Set the data synchronization channel"] = "Den Channel f\195\188r die Datensynchronisation festlegen",
        ["<channel>, <<default>> sets the channel to the default channel, <<addon>> sets the channel to the built-in add on channel"] = "<channel>, <<default>> setzt den Channel auf den Standard-Channel, <<addon>> setzt den Channel auf den integrierten Addon-Channel",
        ["broadcast"] = "senden",
        ["Broadcast the whole database to the designated channel"] = "Die gesamte Datenbank an den festgelegten Channel senden",
        ["Updated version, cleared the trust list, you need to re-enter users"] = "Aktualisierte Version, Zuverl\195\164ssigkeitsliste gel\195\182scht, du musst die Benutzer neu eingeben",
        ["Added the whole guild to the trusted user list"] = "Die gesamte Gilde zur Liste der zuverl\195\164ssigen Benutzer hinzugef\195\188gt",
        ["Added trusted user: "] = "Zuverl\195\164ssigen Benutzer hinzugef\195\188gt: ",
        ["Removed the whole guild from the trusted user list"] = "Die gesamte Gilde von der Liste der zuverl\195\164ssigen Benutzer entfernt",
        ["Removed trusted user: "] = "Vertrauensw\195\188rdigen Benutzer entfernt: ",
        ["Trusting the whole guild"] = "Der gesamten Gilde vertrauen",
        ["Trusted Users: "] = "Vertrauensw\195\188rdige Benutzer: ",
        ["Using the built-in add on channel"] = "Den integrierten Addon-Channel benutzen",
        ["Setting data synchronization channel to default: "] = "Den Channel zur Datensynchronisation auf Standard setzen",
        ["Removing use of add on channel"] = "Die Verwendung des Addon-Channels entfernen",
        ["Setting data synchronization channel to the built-in add on channel"] = "Datensync-Channel auf den integrierten Addon-Channel setzen",
        ["Setting data synchronization channel to: "] = "Datensync-Channel setzen auf: ",
        ["Broadcasting the value database..."] = "Sendung der Datenbank...",
        ["Configured to use the add on channel, but you're not in a guild, use a custom channel instead"] = "Konfiguriert auf die Verwendung des Addon-Channels, aber du bist in keiner Gilde, also verwende einen eigenen Channel stattdessen",
        ["Broadcasting already in progress."] = "Sendung bereits im Gange",
        ["Couldn't join the proper channel for data synchronization"] = "Konnte dem bestimmten Datensynch-Channel nicht beitreten",
	}
end)

ColaMachine = ColaLight:NewModule("ColaMachine", "AceEvent-2.0", "AceConsole-2.0")

ColaMachine.latestId = nil
ColaMachine.isSynching = false
ColaMachine.newValues = nil

local PARSING_PATTERN = ("(%d+)=.-6:([^@]*)@.->?")
local FORMATING_PATTERN = ("%s>%d=6:%d@")
local DEFAULT_CHANNEL_NAME = "lootlinkdatasharing"
local ADDON_MESSAGE_PREFIX = "VendorValueSync"

--[[--------------------------------------------------------------------------------
    OnInitialize, initializes the mod
-----------------------------------------------------------------------------------]]
function ColaMachine:OnInitialize()
    self.db = ColaLight:AcquireDBNamespace("ColaMachine")
    ColaLight:RegisterDefaults("ColaMachine", "account", {
        TrustedUsers = {},
        TrustGuild = true,
        ChannelName = DEFAULT_CHANNEL_NAME,
        UseAddonChannel = true,
        Version = "1.0",
    })
    
    ColaLight.options.args.ColaMachine = {
        name = L["ColaMachine"],
        desc = L["ColaMachine - blows ColaLight cans to pieces"],
        type = "group",
        args = {
            trust = {
                name = L["trust"], type = "text",
                desc = L["Add trusted user"],
                get = false,
                set = function(v) self:TrustUser(v) end,
                usage = L["<name>, <<guild>> adds all in the guild to your trust list"],
            },
            delete = {
                name = L["delete"], type = "text",
                desc = L["Delete trusted user"],
                get = false,
                set = function(v) self:DeleteUser(v) end,
                usage = L["<name>, <<guild>> deletes all in the guild from your trust list"],
            },
            list = {
                name = L["list"], type = "execute",
                desc = L["List trusted users"],
                func = function() self:ListTrustedUsers() end,
            },
            channel = {
                name = L["channel"], type = "text",
                desc = L["Set the data synchronization channel"],
                get = function() if self.db.account.UseAddonChannel then
                        return L["Using the built-in add on channel"]
                    else
                        return self.db.account.ChannelName
                    end
                end,
                set = function(v) self:SetChannel(v) end,
                usage = L["<channel>, <<default>> sets the channel to the default channel, <<addon>> sets the channel to the built-in add on channel"],
            },
            broadcast = {
                name = L["broadcast"], type = "execute",
                desc = L["Broadcast the whole database to the designated channel"],
                func = function() self:Broadcast() end,
            },
        },
        handler = ColaLight,
    }
    
    AceLibrary("AceConsole-2.0"):InjectAceOptionsTable(ColaLight, ColaLight.options.args.ColaMachine)
end

--[[--------------------------------------------------------------------------------
    OnEnable, enable the mod
-----------------------------------------------------------------------------------]]
function ColaMachine:OnEnable()
    -- register for chat messages
    if self.db.account.UseAddonChannel and IsInGuild() then
        self:RegisterEvent("CHAT_MSG_ADDON")
    else
        self:RegisterEvent("CHAT_MSG_CHANNEL")
    end
    
    self:RegisterEvent("ColaLight_NewValueData")
    
    -- version upgrade
    if self.db.account.Version == "1.0" then
        self.db.account.TrustedUsers = {}
        self:Print(L["Updated version, cleared the trust list, you need to re-enter users"])
        self.db.account.Version = "1.1"
    end
end

--[[--------------------------------------------------------------------------------
    OnDisable, disable the mod
-----------------------------------------------------------------------------------]]
function ColaMachine:OnDisable()
    self:UnregisterAllEvents()
end

--[[--------------------------------------------------------------------------------
    Slash command handlers
-----------------------------------------------------------------------------------]]
function ColaMachine:TrustUser(user)
    if user:lower() == "<guild>" then
        self.db.account.TrustGuild = true
        self:Print(L["Added the whole guild to the trusted user list"])
    else
        self.db.account.TrustedUsers[user] = 1
        self:Print(L["Added trusted user: "] .. user)
    end
end

function ColaMachine:DeleteUser(user)
    if user:lower() == "<guild>" then
        self.db.account.TrustGuild = false
        self:Print(L["Removed the whole guild from the trusted user list"])
    else
        for k, v in pairs(self.db.account.TrustedUsers) do
            if k:lower() == user:lower() then
                self.db.account.TrustedUsers[k] = nil
                self:Print(L["Removed trusted user: "] .. user)
            end
        end
    end
end

function ColaMachine:ListTrustedUsers()
    if self.db.account.TrustGuild then
        self:Print(L["Trusting the whole guild"])
    end
    
    local tmp = {}
    for k, v in pairs(self.db.account.TrustedUsers) do
        table.insert(tmp, k)
    end
    
    if select("#", tmp) > 0 then
        self:Print(L["Trusted Users: "] .. table.concat(tmp, ", "))
    end
end

function ColaMachine:SetChannel(channel)
    if channel:lower() == "<default>" then
        self.db.account.UseAddonChannel = false
        self.db.account.ChannelName = DEFAULT_CHANNEL_NAME
        self:RegisterEvent("CHAT_MSG_CHANNEL")
        if self:IsEventRegistered("CHAT_MSG_ADDON") then
            self:UnregisterEvent("CHAT_MSG_ADDON")
            self:Print(L["Removing use of add on channel"])
        end
        self:Print(L["Setting data synchronization channel to default: "] .. self.db.account.ChannelName)
    elseif channel:lower() == "<addon>" then
        self.db.account.UseAddonChannel = true
        self:RegisterEvent("CHAT_MSG_ADDON")
        if self:IsEventRegistered("CHAT_MSG_CHANNEL") then
            self:UnregisterEvent("CHAT_MSG_CHANNEL")
        end
        self:Print(L["Setting data synchronization channel to the built-in add on channel"])
    else
        self.db.account.UseAddonChannel = false
        self.db.account.ChannelName = channel
        self:RegisterEvent("CHAT_MSG_CHANNEL")
        if self:IsEventRegistered("CHAT_MSG_ADDON") then
            self:UnregisterEvent("CHAT_MSG_ADDON")
            self:Print(L["Removing use of add on channel"])
        end
        self:Print(L["Setting data synchronization channel to: "] .. self.db.account.ChannelName)
    end
end

function ColaMachine:Broadcast()
    if not self.isSynching then        
        self:Print(L["Broadcasting the value database..."])
        if self.db.account.UseAddonChannel and not IsInGuild() then
            self:Print(L["Configured to use the add on channel, but you're not in a guild, use a custom channel instead"])
            return
        end
        
        self.isSynching = true
        self:ScheduleRepeatingEvent("SendData", self.SendData, 1, self)
    else
        self:Print(L["Broadcasting already in progress."])
    end
end

--[[--------------------------------------------------------------------------------
    Event Handlers
-----------------------------------------------------------------------------------]]
local player = UnitName("player")
local lastMessage = ""

-- check every chat message for data, add new values if appropriate
function ColaMachine:CHAT_MSG_CHANNEL()
    local message, author, channelName = arg1, arg2, arg9
    
    -- if the message is from the right channel, not the same as the last and not from myself, parse it
    if channelName:lower() == self.db.account.ChannelName:lower() and 
       message ~= lastMessage and 
       author:lower() ~= player:lower() then
        self:ParseMessage(message, author)
    end
    lastMessage = message
end

function ColaMachine:CHAT_MSG_ADDON(prefix, message, distribution, author)
    -- if the message is from ColaMachine and not from myself, parse it 
    if prefix == ADDON_MESSAGE_PREFIX and 
       message ~= lastMessage and
       distribution == "GUILD" and 
       author:lower() ~= player:lower() then
        self:ParseMessage(message, author)
    end
    lastMessage = message
end

function ColaMachine:ColaLight_NewValueData(newValues)
    if not self.isSynching and newValues then
        self.isSynching = true
        self.newValues = newValues
        self:ScheduleRepeatingEvent("SendData", self.SendData, 1, self)
    end
end

--[[--------------------------------------------------------------------------------
    Utilty Functions
-----------------------------------------------------------------------------------]]
function ColaMachine:SendData()
    -- are we synching the whole db or just the new values
    local db
    if self.newValues then
        db = self.newValues
    else
        db = ColaLight.db.account.SellValues
    end
    
    local id, value = next(db, self.latestId)
    
    local channelNumber = 0
    -- get the proper channel
    if not self.db.account.UseAddonChannel then
        channelNumber = GetChannelName(self.db.account.ChannelName)
        if channelNumber == 0 then
            channelNumber = JoinChannelByName(self.db.account.ChannelName)
            if channelNumber == 0 then
                self:Print(L["Couldn't join the proper channel for data synchronization"])
                return
            end
        end
    end
    
    local tmp, message = "", nil
    
    -- if using SendAddonMessage() the whole prefix + message can only be 254, otherwise we
    -- get disconnected, stupid blizzard
    local maxlen = self.db.account.UseAddonChannel and 254 - strlen(ADDON_MESSAGE_PREFIX) or 255
    
    -- loop through the value list
    while strlen(tmp) <= maxlen and id do
        -- save the temporary message
        message = tmp
        -- add the id and value to the temporary message
        tmp = strformat(FORMATING_PATTERN, tmp, id, value)
        
        -- only get then next id-value pair if we the message is below the size limit
        if strlen(tmp) <= maxlen then
            id, value = next(db, id)
        end
    end
    
    -- don't remove the afk flag
    local clearAFK = GetCVar("autoClearAFK")
    SetCVar("autoClearAFK", 0)
    if self.db.account.UseAddonChannel then
        SendAddonMessage(ADDON_MESSAGE_PREFIX, message, "GUILD")
    else
        SendChatMessage(message, "CHANNEL", nil, channelNumber)
    end
    SetCVar("autoClearAFK", clearAFK)
    
    self.latestId = id
    -- are we at the end
    if self.latestId == nil then
        self:CancelScheduledEvent("SendData")
        self.isSynching = false
        
        -- if we were synching new values, reset the new values db
        if self.newValues then
            self.newValues = nil
        end
    end
end

function ColaMachine:ParseMessage(message, author)
    local tmp = {}
    local db = ColaLight.db.account.SellValues
    -- populate a list with the whole guild
    if self.db.account.TrustGuild then
        for i = 1, GetNumGuildMembers(true) do
            name = GetGuildRosterInfo(i)
            tmp[name:lower()] = 1
        end
    end
    
    -- parse the message
    for id, value in strgmatch(message, PARSING_PATTERN) do
        id = tostring(id)
        value = tonumber(value)
        
        -- only add the item if we don't already have it or if it's from a trusted source
        if ColaLight.db.account.SellValues[id] == nil or 
           self.db.account.TrustedUsers[author:lower()] or 
           tmp[author:lower()] then
            db[id] = value
        end
    end
end
